@@ -1,10 +1,8 @@
 import logging
-import uuid
 
 import click
 
 from codecov_cli.fallbacks import CodecovOption, FallbackFieldEnum
-from codecov_cli.helpers.git import GitService
 from codecov_cli.helpers.options import global_options
 from codecov_cli.services.report import create_report_logic
 
@@ -15,6 +13,15 @@ logger = logging.getLogger("codecovcli")
 @click.option(
     "--code", help="The code of the report. If unsure, leave default", default="default"
 )
+@click.option(
+    "-P",
+    "--pr",
+    "--pull-request-number",
+    "pull_request_number",
+    help="Specify the pull request number mannually. Used to override pre-existing CI environment variables",
+    cls=CodecovOption,
+    fallback_field=FallbackFieldEnum.pull_request_number,
+)
 @global_options
 @click.pass_context
 def create_report(
@@ -23,8 +30,9 @@ def create_report(
     code: str,
     slug: str,
     git_service: str,
-    token: uuid.UUID,
+    token: str,
     fail_on_error: bool,
+    pull_request_number: int,
 ):
     enterprise_url = ctx.obj.get("enterprise_url")
     logger.debug(
@@ -41,7 +49,14 @@ def create_report(
         ),
     )
     res = create_report_logic(
-        commit_sha, code, slug, git_service, token, enterprise_url, fail_on_error
+        commit_sha,
+        code,
+        slug,
+        git_service,
+        token,
+        enterprise_url,
+        pull_request_number,
+        fail_on_error,
     )
     if not res.error:
         logger.info(
